__version__ = "1.0.18"
from .client.gelbooru import Gelbooru
from .client.rule34 import Rule34
from .client.tbib import Tbib
from .client.safebooru import Safebooru
from .client.xbooru import Xbooru
from .client.realbooru import Realbooru
from .client.hypnohub import Hypnohub
from .client.danbooru import Danbooru
from .client.atfbooru import Atfbooru
from .client.yandere import Yandere
from .client.konachan import Konachan
from .client.konachan_net import Konachan_Net
from .client.lolibooru import Lolibooru
from .client.e621 import E621
from .client.e926 import E926
from .client.derpibooru import Derpibooru
from .client.furbooru import Furbooru
from .client.behoimi import Behoimi
from .client.paheal import Paheal
from .utils.constant import resolve
